@@ -11,14 +11,20 @@ class WikiIndex(object):
     def __init__(self):
         pass
 
-    def initIndex(self, wiki):
-        raise NotImplementedError()
+    def start(self, wiki):
+        pass
+
+    def init(self, wiki):
+        pass
+
+    def postInit(self):
+        pass
 
     def reset(self, pages):
-        raise NotImplementedError()
+        pass
 
     def update(self, pages):
-        raise NotImplementedError()
+        pass
 
     def search(self, query):
         raise NotImplementedError()
