@@ -3,10 +3,11 @@ import urllib
 import os.path
 import logging
 import jinja2
-from utils import (
+from wikked.formatter import PageFormatter, FormattingContext
+from wikked.utils import (
         PageNotFoundError,
         get_meta_name_and_modifiers, get_absolute_url,
-        html_unescape)
+        flatten_single_metas, html_unescape)
 
 
 logger = logging.getLogger(__name__)
@@ -100,17 +101,19 @@ class PageResolver(object):
         `include` or `query`.
     """
     default_parameters = {
-        '__header': "<ul>\n",
-        '__footer': "</ul>\n",
-        '__item': "<li><a class=\"wiki-link\" data-wiki-url=\"{{url}}\">" +
-            "{{title}}</a></li>\n",
-        '__empty': "<p>No page matches the query.</p>\n"
+        '__header': "\n",
+        '__footer': "\n",
+        '__item': "* [[{{title}}|{{url}}]]\n",
+        '__empty': "No page matches the query.\n"
         }
 
-    def __init__(self, page, ctx=None, parameters=None):
+    def __init__(self, page, ctx=None, parameters=None, page_getter=None,
+                 pages_meta_getter=None):
         self.page = page
         self.ctx = ctx or ResolveContext(page)
         self.parameters = parameters
+        self.page_getter = page_getter or self._getPage
+        self.pages_meta_getter = pages_meta_getter or self._getPagesMeta
         self.output = None
         self.env = None
 
@@ -137,6 +140,15 @@ class PageResolver(object):
             self.output.text = u'<div class="error">%s</div>' % e
             return self.output
 
+    def _getPage(self, url):
+        fields = ['url', 'title', 'path', 'formatted_text', 'local_meta',
+                  'local_links']
+        return self.wiki.db.getPage(url, fields=fields)
+
+    def _getPagesMeta(self):
+        fields = ['url', 'title', 'local_meta']
+        return self.wiki.db.getPages(fields=fields)
+
     def _unsafeRun(self):
         # Create default parameters.
         if not self.parameters:
@@ -148,7 +160,8 @@ class PageResolver(object):
                     'title': self.page.title,
                     'full_title': full_title
                     },
-                '__args': []
+                '__args': [],
+                '__xargs': []
                 }
 
         # Create the output object, so it can be referenced and merged
@@ -189,6 +202,7 @@ class PageResolver(object):
         if self.is_root:
             # Resolve any `{{foo}}` variable references.
             parameters = dict(self.parameters)
+            parameters.update(flatten_single_metas(dict(self.page.getLocalMeta())))
             final_text = self._renderTemplate(final_text, parameters, error_url=self.page.url)
 
             # Resolve link states.
@@ -248,22 +262,25 @@ class PageResolver(object):
             for i, m in enumerate(re.finditer(arg_pattern, args)):
                 value = unicode(m.group('value')).strip()
                 value = html_unescape(value)
-                value = self._renderTemplate(value, parameters, error_url=self.page.url)
+                value = self._renderTemplate(value, self.parameters,
+                                             error_url=self.page.url)
                 if m.group('name'):
                     key = unicode(m.group('name')).lower()
                     parameters[key] = value
                 else:
-                    parameters['__args'].append(value)
+                    parameters['__xargs'].append(value)
+                parameters['__args'].append(value)
 
         # Re-run the resolver on the included page to get its final
         # formatted text.
         try:
-            page = self.wiki.getPage(include_url)
+            page = self.page_getter(include_url)
         except PageNotFoundError:
             raise IncludeError(include_url, self.page.url, "Page not found")
         current_url_trail = list(self.ctx.url_trail)
         self.ctx.url_trail.append(page.url)
-        child = PageResolver(page, self.ctx, parameters)
+        child = PageResolver(page, self.ctx, parameters, self.page_getter,
+                             self.pages_meta_getter)
         child_output = child.run()
         self.output.add(child_output)
         self.ctx.url_trail = current_url_trail
@@ -283,7 +300,7 @@ class PageResolver(object):
         # Parse the query.
         parameters = dict(self.default_parameters)
         meta_query = {}
-        arg_pattern = r"(^|\|)\s*(?P<name>[a-zA-Z][a-zA-Z0-9_\-]+)\s*="\
+        arg_pattern = r"(^|\|)\s*(?P<name>(__)?[a-zA-Z][a-zA-Z0-9_\-]+)\s*="\
             r"(?P<value>[^\|]+)"
         for m in re.finditer(arg_pattern, query):
             key = m.group('name').lower()
@@ -295,7 +312,8 @@ class PageResolver(object):
         # Find pages that match the query, excluding any page
         # that is in the URL trail.
         matched_pages = []
-        for p in self.wiki.getPages():
+        logger.debug("Running page query: %s" % meta_query)
+        for p in self.pages_meta_getter():
             if p.url in self.ctx.url_trail:
                 continue
             for key, value in meta_query.iteritems():
@@ -306,35 +324,60 @@ class PageResolver(object):
                     logger.error("Can't query page '%s' for '%s':" % (p.url, self.page.url))
                     logger.exception(unicode(e.message))
 
+        # We'll have to format things...
+        fmt_ctx = FormattingContext(self.page.url)
+        fmt = PageFormatter()
+
         # No match: return the 'empty' template.
         if len(matched_pages) == 0:
-            return self._valueOrPageText(parameters['__empty'])
+            logger.debug("No pages matched query.")
+            tpl_empty = fmt.formatText(
+                    fmt_ctx, self._valueOrPageText(parameters['__empty']))
+            return tpl_empty
 
         # Combine normal templates to build the output.
-        text = self._valueOrPageText(parameters['__header'])
+        tpl_header = fmt.formatText(
+                fmt_ctx, self._valueOrPageText(parameters['__header']))
+        tpl_footer = fmt.formatText(
+                fmt_ctx, self._valueOrPageText(parameters['__footer']))
+        item_url, tpl_item = self._valueOrPageText(parameters['__item'], with_url=True)
+        tpl_item = fmt.formatText(fmt_ctx, tpl_item)
+
+        text = tpl_header
+        add_trailing_line = tpl_item[-1] == "\n"
         for p in matched_pages:
             tokens = {
                     'url': p.url,
-                    'title': p.title
-                    }
-            tokens.update(p.getLocalMeta())
-            item_url, item_text = self._valueOrPageText(parameters['__item'], with_url=True)
-            text += self._renderTemplate(item_text, tokens, error_url=item_url or self.page.url)
-        text += self._valueOrPageText(parameters['__footer'])
+                    'title': p.title}
+            page_local_meta = flatten_single_metas(dict(p.getLocalMeta()))
+            tokens.update(page_local_meta)
+            text += self._renderTemplate(
+                    tpl_item, tokens, error_url=item_url or self.page.url)
+            if add_trailing_line:
+                # Jinja2 eats trailing new lines... :(
+                text += "\n"
+        text += tpl_footer
 
         return text
 
     def _valueOrPageText(self, value, with_url=False):
-        if re.match(r'^\[\[.*\]\]$', value):
-            include_url = value[2:-2]
+        stripped_value = value.strip()
+        if re.match(r'^\[\[.*\]\]$', stripped_value):
+            include_url = stripped_value[2:-2]
             try:
-                page = self.wiki.getPage(include_url)
+                page = self.page_getter(include_url)
             except PageNotFoundError:
                 raise IncludeError(include_url, self.page.url, "Page not found")
             if with_url:
                 return (page.url, page.text)
             return page.text
 
+        if re.match(r'^__[a-zA-Z][a-zA-Z0-9_\-]+$', stripped_value):
+            meta = self.page.getLocalMeta(stripped_value)
+            if with_url:
+                return (None, meta)
+            return meta
+
         if with_url:
             return (None, value)
         return value
@@ -388,7 +431,7 @@ class PageResolver(object):
         # Recurse into included pages.
         for url in included_urls:
             try:
-                p = self.wiki.getPage(url)
+                p = self.page_getter(url)
             except PageNotFoundError:
                 raise IncludeError(url, page.url, "Page not found")
             if self._isPageMatch(p, name, value, level + 1):
